@@ -1,9 +1,8 @@
 import json
-import os
 from pathlib import Path
 from typing import Dict, Optional
 
-with Path(os.path.dirname(os.path.abspath(__file__)), "data", "aws_managed_policies.json").open(encoding="utf-8") as f:
+with (Path(__file__).absolute().parent / "data" / "aws_managed_policies.json").open(encoding="utf-8") as f:
     _BUNDLED_MANAGED_POLICIES: Dict[str, Dict[str, str]] = json.load(f)
 
 
