@@ -2,15 +2,14 @@ from enum import Enum, auto
 from typing import Any, Dict, List, Optional, Tuple, Union, cast
 
 from samtranslator.metrics.method_decorator import cw_timer
-from samtranslator.model import PropertyType, Resource, ResourceMacro
-from samtranslator.model.iam import IAMRole
-from samtranslator.model.sqs import SQSQueue
-from samtranslator.model.types import is_str, is_type
-from samtranslator.model.eventsources import FUNCTION_EVETSOURCE_METRIC_PREFIX
+from samtranslator.model import Property, PropertyType, Resource, ResourceMacro
 from samtranslator.model.eventbridge_utils import EventBridgeRuleUtils
+from samtranslator.model.eventsources import FUNCTION_EVETSOURCE_METRIC_PREFIX
 from samtranslator.model.exceptions import InvalidEventException
-from samtranslator.model.iam import IAMRolePolicies
+from samtranslator.model.iam import IAMRole, IAMRolePolicies
 from samtranslator.model.scheduler import SchedulerSchedule
+from samtranslator.model.sqs import SQSQueue
+from samtranslator.model.types import IS_BOOL, IS_DICT, IS_STR, PassThrough
 from samtranslator.translator.logical_id_generator import LogicalIdGenerator
 
 
@@ -35,21 +34,22 @@ class SchedulerEventSource(ResourceMacro):
     # same as the original "Schedule" event.
     # See class "Schedule" in samtranslator.model.eventsources.push and samtranslator.model.stepfunctions.events.
     property_types = {
-        "PermissionsBoundary": PropertyType(False, is_str()),
-        "ScheduleExpression": PropertyType(True, is_str()),
-        "FlexibleTimeWindow": PropertyType(False, is_type(dict)),
-        "Name": PropertyType(False, is_str()),
-        "State": PropertyType(False, is_str()),
-        "Description": PropertyType(False, is_str()),
-        "StartDate": PropertyType(False, is_str()),
-        "EndDate": PropertyType(False, is_str()),
-        "ScheduleExpressionTimezone": PropertyType(False, is_str()),
-        "GroupName": PropertyType(False, is_str()),
-        "KmsKeyArn": PropertyType(False, is_str()),
-        "Input": PropertyType(False, is_str()),
-        "RoleArn": PropertyType(False, is_str()),
-        "DeadLetterConfig": PropertyType(False, is_type(dict)),
-        "RetryPolicy": PropertyType(False, is_type(dict)),
+        "PermissionsBoundary": PropertyType(False, IS_STR),
+        "ScheduleExpression": PropertyType(True, IS_STR),
+        "FlexibleTimeWindow": PropertyType(False, IS_DICT),
+        "Name": PropertyType(False, IS_STR),
+        "State": PropertyType(False, IS_STR),
+        "Description": PropertyType(False, IS_STR),
+        "StartDate": PropertyType(False, IS_STR),
+        "EndDate": PropertyType(False, IS_STR),
+        "ScheduleExpressionTimezone": PropertyType(False, IS_STR),
+        "GroupName": PropertyType(False, IS_STR),
+        "KmsKeyArn": PropertyType(False, IS_STR),
+        "Input": PropertyType(False, IS_STR),
+        "RoleArn": PropertyType(False, IS_STR),
+        "DeadLetterConfig": PropertyType(False, IS_DICT),
+        "RetryPolicy": PropertyType(False, IS_DICT),
+        "OmitName": Property(False, IS_BOOL),
     }
 
     # Below are type hints, must maintain consistent with properties_types
@@ -58,23 +58,24 @@ class SchedulerEventSource(ResourceMacro):
     # - pass-through to AWS::Scheduler::Schedule
     ScheduleExpression: str
     FlexibleTimeWindow: Optional[Dict[str, Any]]
-    Name: Optional[str]
-    State: Optional[str]
-    Description: Optional[str]
-    StartDate: Optional[str]
-    EndDate: Optional[str]
-    ScheduleExpressionTimezone: Optional[str]
-    GroupName: Optional[str]
-    KmsKeyArn: Optional[str]
+    Name: Optional[PassThrough]
+    State: Optional[PassThrough]
+    Description: Optional[PassThrough]
+    StartDate: Optional[PassThrough]
+    EndDate: Optional[PassThrough]
+    ScheduleExpressionTimezone: Optional[PassThrough]
+    GroupName: Optional[PassThrough]
+    KmsKeyArn: Optional[PassThrough]
     # - pass-through to AWS::Scheduler::Schedule's Target
-    Input: Optional[str]
-    RoleArn: Optional[str]
+    Input: Optional[PassThrough]
+    RoleArn: Optional[PassThrough]
     DeadLetterConfig: Optional[Dict[str, Any]]
-    RetryPolicy: Optional[Dict[str, Any]]
+    RetryPolicy: Optional[PassThrough]
+    OmitName: Optional[bool]
 
     DEFAULT_FLEXIBLE_TIME_WINDOW = {"Mode": "OFF"}
 
-    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)  # type: ignore
+    @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
     def to_cloudformation(self, **kwargs: Dict[str, Any]) -> List[Resource]:
         """Returns the Scheduler Schedule and an IAM role.
 
@@ -99,7 +100,7 @@ class SchedulerEventSource(ResourceMacro):
         else:
             raise TypeError("Missing required keyword argument: function/resource")
 
-        passthrough_resource_attributes = target.get_passthrough_resource_attributes()  # type: ignore[no-untyped-call]
+        passthrough_resource_attributes = target.get_passthrough_resource_attributes()
 
         resources: List[Resource] = []
 
@@ -124,7 +125,7 @@ class SchedulerEventSource(ResourceMacro):
                 target, target_type, passthrough_resource_attributes, dlq_queue_arn, self.PermissionsBoundary
             )
             resources.append(execution_role)
-            execution_role_arn = execution_role.get_runtime_attr("arn")  # type: ignore[no-untyped-call]
+            execution_role_arn = execution_role.get_runtime_attr("arn")
 
         scheduler_schedule.Target = self._construct_scheduler_schedule_target(target, execution_role_arn, dlq_queue_arn)
 
@@ -139,8 +140,17 @@ class SchedulerEventSource(ResourceMacro):
         if self.State:
             scheduler_schedule.State = self.State
 
-        # Scheduler schedule's Name is a required property
-        scheduler_schedule.Name = self.Name or self.logical_id
+        if self.OmitName:
+            # Originally SAM always generates AWS::Scheduler::Schedule's Name
+            # which caused issues like deploying the same template to multiple stacks
+            # To avoid breaking the backward compatibility, a new property "OmitName"
+            # is introduced and when it is set to True, AWS::Scheduler::Schedule's Name
+            # will not be generated.
+            # https://github.com/aws/serverless-application-model/issues/3109
+            if self.Name:
+                raise InvalidEventException(self.logical_id, "Name cannot be set when OmitName is True")
+        else:
+            scheduler_schedule.Name = self.Name or self.logical_id
 
         # pass-through other properties
         scheduler_schedule.Description = self.Description
@@ -163,10 +173,10 @@ class SchedulerEventSource(ResourceMacro):
     ) -> IAMRole:
         """Constructs the execution role for Scheduler Schedule."""
         if target_type == _SchedulerScheduleTargetType.FUNCTION:
-            policy = IAMRolePolicies.lambda_invoke_function_role_policy(target.get_runtime_attr("arn"), self.logical_id)  # type: ignore[no-untyped-call, no-untyped-call]
+            policy = IAMRolePolicies.lambda_invoke_function_role_policy(target.get_runtime_attr("arn"), self.logical_id)
         elif target_type == _SchedulerScheduleTargetType.STATE_MACHINE:
             policy = IAMRolePolicies.step_functions_start_execution_role_policy(  # type: ignore[no-untyped-call]
-                target.get_runtime_attr("arn"), self.logical_id  # type: ignore[no-untyped-call]
+                target.get_runtime_attr("arn"), self.logical_id
             )
         else:
             raise RuntimeError(f"Unexpected target type {target_type.name}")
@@ -195,7 +205,7 @@ class SchedulerEventSource(ResourceMacro):
         Inspired by https://github.com/aws/serverless-application-model/blob/a25933379e1cad3d0df4b35729ee2ec335402fdf/samtranslator/model/eventsources/push.py#L157
         """
         target_dict: Dict[str, Any] = {
-            "Arn": target.get_runtime_attr("arn"),  # type: ignore[no-untyped-call]
+            "Arn": target.get_runtime_attr("arn"),
             "RoleArn": execution_role_arn,
         }
         if self.Input is not None:
@@ -230,5 +240,5 @@ class SchedulerEventSource(ResourceMacro):
         queue = SQSQueue(queue_logical_id or self.logical_id + "Queue", attributes=passthrough_resource_attributes)
         dlq_resources.append(queue)
 
-        dlq_queue_arn = queue.get_runtime_attr("arn")  # type: ignore[no-untyped-call]
+        dlq_queue_arn = queue.get_runtime_attr("arn")
         return dlq_queue_arn, dlq_resources
